/*

 * SPDX-FileCopyrightText: Copyright (c) 2020 - 2024  NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: BSD-3-Clause
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#pragma once

#include <optix.h>

#include <map>
#include <string>
#include <vector>


class ProgramGroups
{
  public:
    const OptixProgramGroup* data() const;
    unsigned int             size() const;
    const OptixProgramGroup& operator[]( const std::string& name ) const;

    void add( const OptixProgramGroupDesc& programGroupDescriptor, const std::string& name );

  protected:
    ProgramGroups( const OptixDeviceContext context, OptixPipelineCompileOptions pipeOptions, OptixProgramGroupOptions programGroupOptions );

  private:
    const OptixDeviceContext    m_context;
    OptixPipelineCompileOptions m_pipeOptions;
    OptixProgramGroupOptions    m_programGroupOptions;

    std::vector<OptixProgramGroup>      m_programGroups;
    std::map<std::string, unsigned int> m_nameToIndex;
};

class HairProgramGroups : public ProgramGroups
{
  public:
    HairProgramGroups( const OptixDeviceContext context, OptixPipelineCompileOptions pipeOptions,
                       unsigned int buildFlags );

    OptixModule m_shadingModule;
    OptixModule m_whittedModule;
    OptixModule m_quadraticCurveModule;
    OptixModule m_cubicCurveModule;
    OptixModule m_linearCurveModule;
    OptixModule m_catromCurveModule;
};
